local cached = false

hud.add(function(v, p, c)
	if cached then return end
	
	rawset(_G, "clt_font", {monospace = 12})
	for i = 0,255 do -- run once, never again.
		local build = string.format("LTFNT%03d", i)
		if v.patchExists(build) then
			clt_font[i] = v.cachePatch(build)
		end
	end
	
	cached = true
end, "game")

local styles = {
	"FS_SHADOW", // Draws another string right below this (useful for monocolor fonts)
	"FS_MONOSPACE", // Fixed width
	"FS_FIXED" // Enables scaling and places things in the fixed scale, epic!
}
for i = 1,#styles do
	rawset(_G, styles[i], 2^(i-1))
end

hud.add(function(v, p, c)
    if drawStringMkII return end
    
    rawset(_G, "drawStringMkII", function(x, y, str, font, flags, extra) // color, style, flags, align)
		local style,align,color,scale
		if extra ~= nil then
			color = extra.color
			style = extra.style
			//flags = extra.flags
			align = extra.align
			scale = extra.scale
		end
        //if str == "" then str = "-" end // Necessary even?
		if str == true then str = "true" end
		if str == false then str = "false" end
		if type(str) == "number" then str = tostring(str) end
		if type(str) ~= "string" then str = type(str) end
		
		if style == nil then style = 0 end
		if color == nil then color = 0 end
		if scale == nil then scale = FRACUNIT end
		
		if type(color) ~= "table" then
			color = {color}
		end
		
        local dt = {}
        local offset = 0
		local mono = font.monospace
        for i = 1,#str do
            local by = string.byte(str:sub(i,i))
            if font[by] ~= nil then
                dt[#dt+1] = {font[by], offset}
				local ew = font[by].width
				if (style & FS_MONOSPACE) then ew = mono end
				if (style & FS_FIXED) then ew = FixedMul(ew*FRACUNIT, scale) end
				-- FixedMul(ew*FRACUNIT, scale)
                offset = $ + ew
            else
				if (style & FS_FIXED) then mono = FixedMul(mono*FRACUNIT, scale) end
                offset = $ + mono
            end
        end

        local eo = 0
        if align == "center" then
            eo = offset/2
        end
        if align == "right" then
            eo = offset
        end
		
		local colormaps = {}
		for i = 1,#color do
			colormaps[i] = v.getColormap(-1, color[i])
		end
		
        for i = 1,#dt do
            local ix = dt[i]
			
			if (style & FS_SHADOW) then
				--v.draw(x+ix[2]-eo+1, y+1, ix[1], flags, colormaps[2])
			end
            --v.draw(x+ix[2]-eo, y, ix[1], flags, colormaps[1])
			if (style & FS_FIXED) then
				v.drawScaled(x+ix[2]-eo, y, scale, ix[1], flags, colormaps[1])
			else
				v.draw(x+ix[2]-eo, y, ix[1], flags, colormaps[1])
			end
			--v.drawScaled(x+ix[2]-eo, y, ix[1], flags, colormaps[1])
        end
    end)
end)

-- solo version
local function L_LevelNameWidth(str, mono)
	local offset = 0
	for i = 1,#str do
		local by = string.byte(str:sub(i,i))
		if clt_font[by] ~= nil then
			local ew = clt_font[by].width
			if (mono) then ew = 12 end
			offset = $ + ew
		else
			offset = $ + 12
		end
	end
	
	return offset
end

// Hidden/hellmap?
local function ReadHidden(mapid)
	if (mapheaderinfo[mapid].menuflags & LF2_HIDEINMENU) == 1 then return true else return end
end

local function L_GetGametypeColor() -- hell
	if G_BattleGametype() and not ReadHidden(gamemap) then
		return 126, "3"
	elseif G_RaceGametype() and not ReadHidden(gamemap) then
		return 214, "2"
	else
		return 85, "4" -- hell
	end
end

// Note: NoZone is checked via the LF_NOZONE level flag.
local function ReadNoZone(mapid)
	if (mapheaderinfo[mapid].levelflags & LF_NOZONE) == 16 then return true end
end

local cv_alignment = CV_RegisterVar({
	name = "stl_alignment",
	defaultvalue = "Bottom",
	possiblevalue = {Top = 1, Center = 2, Bottom = 0},
	flags = CV_SHOWMODIF
})

COM_AddCommand("stl_resetfontcache", function(p, ...)
	if not cached then CONS_Printf(p, "The font isn't cached yet!") return end
	
	rawset(_G, "clt_font", nil) -- whoops
	cached = false
	CONS_Printf(p, "Successfully reset font cache.")
end)

local function L_DrawLevelTitle(levelname, subtitle, zonename, actnum, v)
	local gtcolor, gtoffset = L_GetGametypeColor()
	local bary = 163
	local intdupx, fixeddupx = v.dupx()
	local dupcalc = v.width()/intdupx
	local intdupy, fixeddupy = v.dupy()
	local dupcalcy = v.height()/intdupy
	local sub_ = 0
	
	if cv_alignment.value == 1 then
		bary = 63
	elseif cv_alignment.value == 2 then
		bary = 100
	end

	if splitscreen then bary = 100 end
	
	if leveltime > 113 then return end -- no more.
	
	local lvlw = L_LevelNameWidth(levelname)
	local lvlxpos = 160 - (lvlw/2)
	
	local ttlnumxpos = lvlxpos + lvlw
	local zonexpos = ttlnumxpos
	if not ReadNoZone(gamemap) then
		zonexpos = $ - L_LevelNameWidth(zonename)
	end
	
	if lvlxpos < 0 then lvlxpos = 0 end -- simple enough, right?
	
	if leveltime > 105 then -- damn. math
		local count = (113 - leveltime)
		sub_ = dupcalc
		
		while count > 0 do
			sub_ = $ >> 1
			count = $ - 1
		end
		sub_ = -sub_
	end
	
	dupcalc = (dupcalc - 160)>>1
	v.drawFill(sub_ - dupcalc, bary+9, ttlnumxpos+dupcalc + 1, 2, 31)
	v.draw(sub_ + ttlnumxpos, bary, v.cachePatch("STGTTLD1"))
	v.drawFill(sub_ - dupcalc, bary, ttlnumxpos+dupcalc, 10, gtcolor)
	v.draw(sub_ + ttlnumxpos, bary, v.cachePatch("STGTTLD"..tostring(gtoffset)))
	if zonename == "ZONE" or zonename == "" then
		v.drawString(sub_ + zonexpos - 8, bary+1, subtitle, V_ALLOWLOWERCASE, "right")
	elseif subtitle:len() + zonename:len() > 24
		v.drawString(sub_ + zonexpos - v.stringWidth(subtitle, 0, "thin"), bary+1, subtitle, V_ALLOWLOWERCASE, "thin")
	else
		v.drawString(sub_ + zonexpos - 8, bary+1, subtitle, V_ALLOWLOWERCASE, "right")
	end
	
	ttlnumxpos = $ + sub_
	lvlxpos = $ + sub_
	zonexpos = $ + sub_
	
	drawStringMkII(lvlxpos, bary-18, levelname, clt_font, 0)
	drawStringMkII(zonexpos, bary+6, zonename, clt_font, 0)
	if actnum ~= "" then
		drawStringMkII(ttlnumxpos+12, bary+6, actnum, clt_font, 0) -- might change this
	end
end

local function L_CheckIfEmpty(string, r)
	if string == "" or string == nil then
		return r
	end
	return string
end

hud.add(function(v, p, c)
	local lvlttl = L_CheckIfEmpty(mapheaderinfo[gamemap].lvlttl, "NO NAME")
	local subttl = L_CheckIfEmpty(mapheaderinfo[gamemap].subttl, "Empty Subtitle")
	local zonttl = L_CheckIfEmpty(mapheaderinfo[gamemap].zonttl, "ZONE")
	local actnum = L_CheckIfEmpty(mapheaderinfo[gamemap].actnum, "")
	if ReadNoZone(gamemap) and mapheaderinfo[gamemap].zonttl == "" then
		zonttl = ""
	end
	
	if encoremode and mapheaderinfo[gamemap].encorettl then
		lvlttl = mapheaderinfo[gamemap].encorettl
	end
	if encoremode and mapheaderinfo[gamemap].encoresub then
		subttl = mapheaderinfo[gamemap].encoresub
	end
	if encoremode and mapheaderinfo[gamemap].encorezone then
		zonttl = mapheaderinfo[gamemap].encorezone
	end
	
	L_DrawLevelTitle(lvlttl:upper(), subttl, zonttl:upper(), actnum:upper(), v)
end, "game")
hud.disable("stagetitle") -- gone byebye